/*
    Copyright (c) 2009 by Joseph Devietti (devietti@cs.washington.edu).

    This file is part of my DMP ASPLOS 2009 Presentation (DAP).

    DAP is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DAP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DAP.  If not, see <http://www.gnu.org/licenses/>.
*/ 

﻿package  {
	
	import com.boostworthy.geom.Path;
	import flash.display.DisplayObject;
	import flash.display.DisplayObjectContainer;
	import flash.display.Sprite;
	import flash.errors.IllegalOperationError;
	import flash.geom.*;
	
	/**
	 * Class that automatically manages wipe transitions.
	 */
	public class Wiper {
		
		public static const WIPE_UP:int = 1;
		public static const WIPE_DOWN:int = 2;
		public static const WIPE_LEFT:int = 3;
		public static const WIPE_RIGHT:int = 4;
		
		
		/** Sets up a wipe transition for the given DisplayObject, wiping in the specified 
		 * direction and making the object (dis)appear.
		 * Returns an object with three fields: 
		 *   mask - a reference to the mask for DisplayObject
		 *   x - the target x value to move the mask to 
		 *   y - the target y value to move the mask to
		 * The caller must setup a move transition to the specified (wipe_x,wipe_y) coords 
		 * to actually make the wipe happen. 
		 * @param d the DisplayObject to wipe 
		 * @param direction the direction in which to wipe 
		 * @param appear if true, wipe *in* the object, else wipe it out
		 * @param debug debug the wipe by rendering the mask as a visible shape */
		public static function wipe(d:DisplayObjectContainer, direction:int, appear:Boolean, debug:Boolean=false):Object {
			var mask:Sprite = new Sprite();
			mask.graphics.lineStyle();
			mask.graphics.beginFill( 0x0 ); // doesn't matter what color this is
			
			// TODO: get wiping to work for objects where 0,0 isn't TL corner. Need to figure out bounds in global coord space
			//trace( d.getBounds(d).toString() );
			//trace( d.getRect(d).toString() );
			
			var bounds:Rectangle = d.getBounds(d);
			mask.graphics.drawRect( 0, 0, bounds.width, bounds.height );
			mask.graphics.endFill();
			if ( !debug ) {
				d.addChild( mask );
				d.mask = mask;
			} else {
				trace( "DEBUGGING!" );
				trace( bounds.toString() );
			}
						
			var ret:Object = { mask:mask, x:undefined, y:undefined };
			
			if ( appear ) {
				// mask needs to finish right on top of masked object
				ret.x = bounds.x;
				ret.y = bounds.y;
				
				switch (direction) {
				case WIPE_DOWN:
					mask.x = bounds.x;
					mask.y = bounds.y - bounds.height;
					break;
				case WIPE_UP:
					mask.x = bounds.x;
					mask.y = bounds.y + bounds.height;
					break;
				case WIPE_LEFT:
					mask.x = bounds.x + bounds.width;
					mask.y = bounds.y;
					break;
				case WIPE_RIGHT:
					mask.x = bounds.x - bounds.width;
					mask.y = bounds.y;
					break;
				default:
					throw new ArgumentError( "Invalid direction: "+direction );
				}
				
			} else { // disappear
				// start out with entire object visible
				mask.x = bounds.x;
				mask.y = bounds.y;
				
				switch (direction) {
				case WIPE_DOWN:
					ret.x = bounds.x;
					ret.y = bounds.y + bounds.height;
					break;
				case WIPE_UP:
					ret.x = bounds.x;
					ret.y = bounds.y - bounds.height;
					break;
				case WIPE_LEFT:
					ret.x = bounds.x - bounds.width;
					ret.y = bounds.y;
					break;
				case WIPE_RIGHT:
					ret.x = bounds.x + bounds.width;
					ret.y = bounds.y;
					break;
				default:
					throw new ArgumentError( "Invalid direction: "+direction );
				}
			}

			return ret;
		}
		
		/** Sets up a wipe transition for the given DisplayObject, wiping along the specified path.
		 * The caller must setup a PathTween in order to actually make the wipe happen. 
		 * @param d the DisplayObject to wipe 
		 * @param p the Path along which to wipe
		 * @param debug debug the wipe by rendering the mask as a visible shape
		 * @return the mask object that needs to be animated */
		public static function pathWipe(d:DisplayObjectContainer, p:Path, debug:Boolean=false):Sprite {
			var mask:Sprite = new Sprite();
			var bounds:Rectangle = d.getBounds( d );
			
			//trace( "bounds: " + bounds.toString() );
			
			mask.graphics.lineStyle();
			mask.graphics.beginFill( 0x0 );
			// need to have (0,0) at the rightmost edge of the mask
			var size:Number = Math.max( Point.distance(bounds.topLeft,bounds.bottomRight),
				Point.distance(new Point(bounds.left,bounds.bottom),new Point(bounds.right,bounds.top)) );
			mask.graphics.drawRect( (-size)/*-2*/, -size/2, size, size );
			mask.graphics.endFill();
			
			// draw registration point for debugging
			//mask.graphics.beginFill( 0x00ff00 );
			//mask.graphics.drawCircle( 0, 0, 1 );
			
			d.addChild( mask );
			if ( !debug ) {
				d.mask = mask;
			}
			return mask;
		}
		
	}
	
}